#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Unity Mail, functions for working with configuration file
# Authors: Dmitry Shachnev <mitya57@gmail.com>
#          Robert Tari <robert.tari@gmail.com>
# License: GNU GPL 3 or higher; http://www.gnu.org/licenses/gpl.html

from gi.repository import GLib
from configparser import RawConfigParser
from os.path import join

config_dir = GLib.get_user_config_dir()
config_file_path = join(config_dir, 'unity-mail.conf')

class ConfigFile(RawConfigParser):

    def __init__(self, filename):

        RawConfigParser.__init__(self, strict=False)
        self.optionxform = str
        self.read(filename)

    def get_str_with_default(self, item, defaultvalue = '', section = 'General'):

        if self.has_option(section, item):
            return self.get(section, item)
        else:
            return defaultvalue

    def get_bool_with_default(self, item, defaultvalue, section = 'General'):

        if self.has_option(section, item):
            return self.getboolean(section, item)
        else:
            return defaultvalue

    def get_int_with_default(self, item, defaultvalue, section = 'General'):

        if self.has_option(section, item):
            return self.getint(section, item)
        else:
            return defaultvalue



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Unity Mail, the Preferences Dialog
# Authors: Dmitry Shachnev <mitya57@gmail.com>
#          Robert Tari <robert.tari@gmail.com>
# License: GNU GPL 3 or higher; http://www.gnu.org/licenses/gpl.html

import UnityMail.imaplib2 as imaplib
import os.path
from socket import error as socketerror
from gi.repository import Gtk, GdkPixbuf, Gdk
from UnityMail import g_oSettings
from UnityMail.config import ConfigFile, config_file_path
from UnityMail.application import APPVERSION

MESSAGEACTION = {'OPEN': 1, 'MARK': 2, 'ASK': 3}

def utf7dec(lstInput):

    if not isinstance(lstInput, bytes):
        return lstInput

    lstResult = []
    lstBytes = bytearray()

    for nChar in lstInput:

        if nChar == b'&'[0] and not lstBytes:

            lstBytes.append(nChar)

        elif nChar == b'-'[0] and lstBytes:

            if len(lstBytes) == 1:
                lstResult.append('&')
            else:
                lstResult.append((b'+' + lstBytes[1:].replace(b',', b'/') + b'-').decode('utf-7'))

            lstBytes = bytearray()

        elif lstBytes:

            lstBytes.append(nChar)

        else:

            lstResult.append(chr(nChar))

    if lstBytes:
        lstResult.append((b'+' + lstBytes[1:].replace(b',', b'/') + b'-').decode('utf-7'))

    return ''.join(lstResult)

class ComboEntry(Gtk.ButtonBox):

    def __init__(self, strText, strId, **kwargs):

        Gtk.ButtonBox.__init__(self, orientation=Gtk.Orientation.HORIZONTAL, layout_style=Gtk.ButtonBoxStyle.EXPAND, **kwargs)

        self.oComboBoxText = Gtk.ComboBoxText()
        self.oComboBoxText.append('HTTP:', _('Web page'))
        self.oComboBoxText.append('Exec:', _('Command'))
        self.oEntry = Gtk.Entry(text=strText[5:] if strText.startswith('Exec:') else strText)
        self.oEntry.set_icon_from_icon_name(Gtk.EntryIconPosition.SECONDARY, 'gtk-revert-to-saved-ltr')
        self.oEntry.connect('icon-press', self.onReset)
        self.oComboBoxText.set_active_id('Exec:' if strText.startswith('Exec:') else 'HTTP:')
        self.pack_start(self.oComboBoxText, False, False, 0)
        self.pack_start(self.oEntry, True, True, 0)
        self.set_homogeneous(False)
        self.strId = strId

    def onReset(self, oWidget, nPosition, oEvent):

        strText = g_oSettings.get_default_value(self.strId).get_string();
        self.oEntry.set_text(strText[5:] if strText.startswith('Exec:') else strText)
        self.oComboBoxText.set_active_id('Exec:' if strText.startswith('Exec:') else 'HTTP:')

    def getText(self):

        return ('Exec:' if self.oComboBoxText.get_active_id() == 'Exec:' else '') + self.oEntry.get_text()

class FileChooserButtonEx(Gtk.ButtonBox):

    def __init__(self, strFilename, **kwargs):

        Gtk.ButtonBox.__init__(self, orientation=Gtk.Orientation.HORIZONTAL, layout_style=Gtk.ButtonBoxStyle.EXPAND, **kwargs)

        oButtonClear = Gtk.Button.new_from_icon_name('gtk-clear', Gtk.IconSize.BUTTON)
        oButtonClear.connect('clicked', self.onClear)
        oButtonOpen = Gtk.Button()
        oButtonOpen.connect('clicked', self.onOpen)
        self.oLabel = Gtk.Label(os.path.basename(strFilename) or _('(None)'), xalign=0)
        oImage = Gtk.Image.new_from_icon_name('gtk-open', Gtk.IconSize.BUTTON)
        oBox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        oBox.pack_start(oImage, False, False, 0)
        oBox.pack_start(self.oLabel, True, True, 0)
        oButtonOpen.add(oBox)
        self.pack_start(oButtonOpen, True, True, 0)
        self.pack_start(oButtonClear, False, False, 0)
        self.set_homogeneous(False)
        self.strFilename = strFilename

    def onClear(self, oWidget):

        self.strFilename = ''
        self.oLabel.set_text(_('(None)'))

    def onOpen(self, oWidget):

        oDlg = Gtk.FileChooserDialog(None, oWidget.get_toplevel(), 0, (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.APPLY))

        if self.strFilename:
            oDlg.set_filename(self.strFilename)

        nResponse = oDlg.run()

        if nResponse == Gtk.ResponseType.APPLY:

            self.strFilename = oDlg.get_filename()
            self.oLabel.set_text(os.path.basename(self.strFilename))

        oDlg.destroy()

    def getFilename(self):

        return self.strFilename

class PreferencesDialog(Gtk.Dialog):

    def __init__(self):

        self.bInit = False
        self.initConfig()
        Gtk.Dialog.__init__(self, _('Unity Mail Preferences'), None, 0, (Gtk.STOCK_CONNECT, 100, Gtk.STOCK_APPLY, Gtk.ResponseType.APPLY, Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL))
        self.set_icon_name('unity-mail')
        self.connect('destroy', lambda w: Gtk.main_quit())
        self.connect('response', self.onResponse)
        self.set_position(Gtk.WindowPosition.CENTER)
        self.set_property('width-request', 640)
        self.set_property('height-request', 480)
        self.oNotebook = Gtk.Notebook(vexpand=True, margin_left=5, margin_top=5, margin_right=5)
        self.oNotebook.append_page(self.pageAccounts(), Gtk.Label(_('Accounts')))
        self.oNotebook.append_page(self.pageLinks(), Gtk.Label(_('Links')))
        self.oNotebook.append_page(self.pageOptions(), Gtk.Label(_('Options')))
        self.oNotebook.append_page(self.pageSupport(), Gtk.Label(_('Support')))
        self.oNotebook.append_page(self.pageAbout(), Gtk.Label(_('About')))
        oContentArea = self.get_content_area()
        oContentArea.set_property('vexpand', True)
        oContentArea.add(self.oNotebook)
        self.oButtonConnect = self.get_widget_for_response(100)
        self.oButtonApply = self.get_widget_for_response(Gtk.ResponseType.APPLY)
        self.lstDicts = [{'Host': 'imap.gmail.com', 'Port': '993', 'Login': '', 'Passwd': '', 'Folders': 'INBOX'}]
        self.set_keep_above(True)
        self.show_all()
        self.bInit = True
        self.nIndex = 0

    def onResponse(self, oWidget, nResponse):

        if nResponse == 100:

            self.oNotebook.set_current_page(0)
            self.oListStore.clear()
            self.getFolderList(True)
            self.updateUI()

    def getFolderList(self, bAutoselect):

        try:

            oImap = None

            try:

                oImap = imaplib.IMAP4_SSL(self.EntryHost.get_text(), int(self.EntryPort.get_text()))

            except Exception:

                oImap = imaplib.IMAP4(self.EntryHost.get_text(), int(self.EntryPort.get_text()))
                oImap.starttls()

            oImap.login(self.EntryLogin.get_text(), self.EntryPassword.get_text())

            for f in oImap.list()[1]:

                flags, b, c = utf7dec(f).partition(' ')
                separator, b, strName = c.partition(' ')
                strName = strName.replace('"', '').replace('/ ', '')

                if strName != '[Gmail]' and 'All Mail' not in strName:
                    self.oListStore.append([strName, strName.replace('[Gmail]/', ''), strName.upper() == 'INBOX' if bAutoselect else False])

            oImap.logout()

        except (imaplib.IMAP4.error, socketerror) as oError:

            oDlg = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.ERROR, Gtk.ButtonsType.CLOSE, '')

            strError = str(oError)

            if strError.startswith("b'"):
                strError = strError[1:]

            if 'https://support.google.com/mail/accounts/answer/78754' in strError:

                oDlg.set_markup( _('Please visit the following link and enable access for less secure apps:') + '\n\n<a href="https://www.google.com/settings/security/lesssecureapps">https://www.google.com/settings/security/lesssecureapps</a>')
                oDlg.set_property('message_type', Gtk.MessageType.INFO)
                oDlg.set_title(_('Access blocked by Google'))

            else:

                oDlg.set_property('text', _('Failed to connect to mail account. The returned error was:') + '\n\n' + strError)
                oDlg.set_title(_('Connection failure'))

            oDlg.run()
            oDlg.destroy()

    def initConfig(self):

        self.nMaxCount = g_oSettings.get_int('max-item-count')
        self.bEnableNotifications = g_oSettings.get_boolean('enable-notifications')
        self.bPlaySound = g_oSettings.get_boolean('enable-sound')
        self.bHideCount = g_oSettings.get_boolean('hide-messages-count')
        self.strCommand = g_oSettings.get_string('exec-on-receive')
        self.strCustomSound = g_oSettings.get_string('custom-sound')
        self.bMergeConversation = g_oSettings.get_boolean('merge-messages')
        self.nMessageAction = g_oSettings.get_enum('message-action')
        self.dctURLs = {}
        self.dctURLs['Home'] = g_oSettings.get_string('home')
        self.dctURLs['Compose'] = g_oSettings.get_string('compose')
        self.dctURLs['Inbox'] = g_oSettings.get_string('inbox')
        self.dctURLs['Sent'] = g_oSettings.get_string('sent')

    def pageAccounts(self):

        acclabel = Gtk.Label(xalign=0, hexpand=True)
        acclabel.set_markup('<b>'+_('Choose an account')+'</b>')
        self.sb = Gtk.SpinButton.new_with_range(1, 1, 1)
        self.sb.set_numeric(True)
        self.sb.set_update_policy(Gtk.SpinButtonUpdatePolicy.IF_VALID)
        self.sb.connect('value-changed', self.onAccountChanged)
        self.addbtn = Gtk.Button.new_with_label(_('Add'))
        self.addbtn.connect('clicked', self.onAddAccount)
        self.rmbtn = Gtk.Button.new_with_label(_('Remove'))
        self.rmbtn.connect('clicked', self.onRemoveAccount)
        accbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=5, hexpand=True)
        accbox.pack_start(self.sb, True, True, 0)
        accbox.pack_end(self.rmbtn, False, False, 0)
        accbox.pack_end(self.addbtn, False, False, 0)
        self.EntryHost = Gtk.Entry(hexpand=True)
        self.EntryHost.connect('changed', lambda w: self.updateUI())
        self.EntryPort = Gtk.Entry(hexpand=True)
        self.EntryPort.connect('changed', lambda w: self.updateUI())
        self.EntryLogin = Gtk.Entry(hexpand=True)
        self.EntryLogin.connect('changed', lambda w: self.updateUI())
        self.EntryPassword = Gtk.Entry(visibility=False, caps_lock_warning=True, hexpand=True)
        self.EntryPassword.connect('changed', lambda w: self.updateUI())
        srvlabel = Gtk.Label(xalign=0, margin_top=5, hexpand=True)
        srvlabel.set_markup('<b>'+_('Server data')+'</b>')
        infolabel = Gtk.Label(xalign=0, margin_top=5, hexpand=True)
        infolabel.set_markup('<b>'+_('Account data')+'</b>')
        self.oListStore = Gtk.ListStore(str, str, bool)
        oTreeView = Gtk.TreeView(self.oListStore, headers_visible=False, activate_on_single_click=True, margin_left=5, margin_top=5, margin_right=5, margin_bottom=5)
        oTreeView.connect('row-activated', self.onFolderActivated)
        oTreeViewColumnBool = Gtk.TreeViewColumn('bool', Gtk.CellRendererToggle(), active=2)
        oTreeViewColumnBool.get_cells()[0].set_property('xalign', 1.0)
        oTreeView.append_column(Gtk.TreeViewColumn('folder', Gtk.CellRendererText(), text=1))
        oTreeView.append_column(oTreeViewColumnBool)
        oFrame = Gtk.ScrolledWindow(shadow_type=Gtk.ShadowType.IN, hexpand=True, vexpand=True)
        oFrame.add(oTreeView)
        oGrid = Gtk.Grid(row_spacing=5, column_spacing=10, vexpand=True)
        oGrid.attach(srvlabel, 0, 0, 2, 1)
        oGrid.attach(Gtk.Label(_('Host:'), xalign=0), 0, 1, 1, 1)
        oGrid.attach(self.EntryHost, 1, 1, 1, 1)
        oGrid.attach(Gtk.Label(_('Port:'), xalign=0), 0, 2, 1, 1)
        oGrid.attach(self.EntryPort, 1, 2, 1, 1)
        oGrid.attach(infolabel, 0, 3, 2, 1)
        oGrid.attach(Gtk.Label(_('Login:'), xalign=0), 0, 4, 1, 1)
        oGrid.attach(self.EntryLogin, 1, 4, 1, 1)
        oGrid.attach(Gtk.Label(_('Password:'), xalign=0), 0, 5, 1, 1)
        oGrid.attach(self.EntryPassword, 1, 5, 1, 1)
        oGrid.attach(Gtk.Label(_('Folders:'), xalign=0), 0, 6, 1, 1)
        oGrid.attach(oFrame, 1, 6, 1, 1)
        page = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=5, border_width=10, vexpand=True)
        page.add(acclabel)
        page.add(accbox)
        page.add(oGrid)

        return page

    def pageLinks(self):

        self.oEntryHome = ComboEntry(self.dctURLs['Home'], 'home', hexpand=True)
        self.oEntryCompose = ComboEntry(self.dctURLs['Compose'], 'compose', hexpand=True)
        self.oEntryInbox = ComboEntry(self.dctURLs['Inbox'], 'inbox', hexpand=True)
        self.oEntrySent = ComboEntry(self.dctURLs['Sent'], 'sent', hexpand=True)
        oGrid = Gtk.Grid(row_spacing=5, column_spacing=10)
        oGrid.attach(Gtk.Label(_('Home:'), xalign=0), 0, 0, 1, 1)
        oGrid.attach(self.oEntryHome, 1, 0, 1, 1)
        oGrid.attach(Gtk.Label(_('Compose:'), xalign=0), 0, 1, 1, 1)
        oGrid.attach(self.oEntryCompose, 1, 1, 1, 1)
        oGrid.attach(Gtk.Label(_('Inbox:'), xalign=0), 0, 2, 1, 1)
        oGrid.attach(self.oEntryInbox, 1, 2, 1, 1)
        oGrid.attach(Gtk.Label(_('Sent:'), xalign=0), 0, 3, 1, 1)
        oGrid.attach(self.oEntrySent, 1, 3, 1, 1)
        page = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=5, border_width=10)
        page.add(oGrid)

        return page

    def pageOptions(self):

        self.oComboMessageAction = Gtk.ComboBoxText()
        self.oComboMessageAction.append(str(MESSAGEACTION['MARK']), _('Mark message as read'))
        self.oComboMessageAction.append(str(MESSAGEACTION['OPEN']), _('Open message in browser/Execute command'))
        self.oComboMessageAction.append(str(MESSAGEACTION['ASK']), _('Ask me what to do'))
        self.oComboMessageAction.set_active_id(str(self.nMessageAction))
        self.oSwitchMerge = Gtk.Switch(active=self.bMergeConversation, halign=Gtk.Align.END)
        self.notifyswitch = Gtk.Switch(active=self.bEnableNotifications, halign=Gtk.Align.END)
        self.sndswitch = Gtk.Switch(active=self.bPlaySound, halign=Gtk.Align.END)
        self.sndswitch.connect('notify::active', self.onSoundSwitchActivate)
        self.hcswitch = Gtk.Switch(active=self.bHideCount, halign=Gtk.Align.END)
        self.commandchooser = FileChooserButtonEx(self.strCommand)
        self.sndchooser = FileChooserButtonEx(self.strCustomSound)
        oGrid = Gtk.Grid(row_spacing=5, column_spacing=10)
        oGrid.attach(Gtk.Label(_('Enable notifications:'), xalign=0), 0, 0, 1, 1)
        oGrid.attach(self.notifyswitch, 1, 0, 1, 1)
        oGrid.attach(Gtk.Label(_('Play sound on messages receive:'), xalign=0), 0, 1, 1, 1)
        oGrid.attach(self.sndswitch, 1, 1, 1, 1)
        oGrid.attach(Gtk.Label(_('Merge messages from the same conversation:'), xalign=0), 0, 2, 1, 1)
        oGrid.attach(self.oSwitchMerge, 1, 2, 1, 1)
        oGrid.attach(Gtk.Label(_('Hide count when it is zero:'), xalign=0), 0, 3, 1, 1)
        oGrid.attach(self.hcswitch, 1, 3, 1, 1)
        oGrid.attach(Gtk.Label(_('When a message is activated:'), xalign=0), 0, 4, 1, 1)
        oGrid.attach(self.oComboMessageAction, 1, 4, 1, 1)
        commandbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=10)
        commandbox.add(Gtk.Label(_('Execute this command on messages receive:'), xalign=0))
        commandbox.pack_end(self.commandchooser, True, True, 0)

        if self.commandchooser.get_allocated_width() < 180:
            self.commandchooser.set_size_request(180, 0)

        self.sndbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=10, sensitive=self.bPlaySound)
        self.sndbox.add(Gtk.Label(_('Custom sound to play:'), xalign=0))
        self.sndbox.pack_end(self.sndchooser, True, True, 0)

        if self.sndchooser.get_allocated_width() < 180:
            self.sndchooser.set_size_request(180, 0)

        page = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=5, border_width=10)
        page.add(oGrid)
        page.add(commandbox)
        page.add(self.sndbox)

        return page

    def pageSupport(self):

        oGrid = Gtk.Grid(border_width=10, row_spacing=2)
        oGrid.attach(Gtk.Label(_('Report a bug/Request a feature'), xalign=0), 0, 0, 1, 1)
        oGrid.attach(Gtk.Label('<a href="https://bugs.launchpad.net/unity-mail/+filebug">https://bugs.launchpad.net/unity-mail/+filebug</a>', xalign=0, use_markup=True, margin_bottom=5), 0, 1, 1, 1)
        oGrid.attach(Gtk.Label(_('Ask a question'), xalign=0), 0, 2, 1, 1)
        oGrid.attach(Gtk.Label('<a href="https://answers.launchpad.net/unity-mail/+addquestion">https://answers.launchpad.net/unity-mail/+addquestion</a>', xalign=0, use_markup=True, margin_bottom=5), 0, 3, 1, 1)
        oGrid.attach(Gtk.Label(_('Help translate'), xalign=0), 0, 4, 1, 1)
        oGrid.attach(Gtk.Label('<a href="https://translations.launchpad.net/unity-mail/trunk">https://translations.launchpad.net/unity-mail/trunk</a>', xalign=0, use_markup=True, margin_bottom=5), 0, 5, 1, 1)
        oGrid.attach(Gtk.Label(_('Project page'), xalign=0), 0, 6, 1, 1)
        oGrid.attach(Gtk.Label('<a href="https://launchpad.net/unity-mail">https://launchpad.net/unity-mail</a>', xalign=0, use_markup=True, margin_bottom=5), 0, 7, 1, 1)
        oGrid.attach(Gtk.Label(_('Source code'), xalign=0), 0, 8, 1, 1)
        oGrid.attach(Gtk.Label('<a href="http://bazaar.launchpad.net/~robert-tari/unity-mail/trunk/files">http://bazaar.launchpad.net/~robert-tari/unity-mail/trunk/files</a>', xalign=0, use_markup=True, margin_bottom=5), 0, 9, 1, 1)
        oGrid.attach(Gtk.Label(_('Repository'), xalign=0), 0, 10, 1, 1)
        oGrid.attach(Gtk.Label('<a href="https://launchpad.net/~robert-tari/+archive/ubuntu/main">https://launchpad.net/~robert-tari/+archive/ubuntu/main</a>', xalign=0, use_markup=True, margin_bottom=5), 0, 11, 1, 1)
        oGrid.attach(Gtk.Label(_('Home page'), xalign=0), 0, 12, 1, 1)
        oGrid.attach(Gtk.Label('<a href="https://tari.in/www/software/unitymail/">https://tari.in/www/software/unitymail/</a>', xalign=0, use_markup=True, margin_bottom=5), 0, 13, 1, 1)

        return oGrid

    def pageAbout(self):

        oBox = Gtk.Box()
        oAboutDialog = Gtk.AboutDialog()
        oAboutDialog.set_license_type(Gtk.License.GPL_3_0)
        oAboutDialog.set_program_name('Unity Mail')
        oAboutDialog.set_copyright('Robert Tari 2016-2017')
        oAboutDialog.set_comments(_('Mail notifications and counter for Unity/MATE/Xfce/LXDE/etc.'))
        oAboutDialog.set_authors(['Dmitry Shachnev &lt;mitya57@gmail.com&gt;', 'Robert Tari &lt;robert.tari@gmail.com&gt;'])
        oAboutDialog.set_translator_credits(_('translator-credits'))
        oAboutDialog.set_version(APPVERSION)
        oAboutDialog.set_website('https://tari.in/www/software/unitymail/')
        oAboutDialog.set_website_label('https://tari.in/www/software/unitymail/')
        oAboutDialog.set_logo(GdkPixbuf.Pixbuf().new_from_file('/usr/share/icons/hicolor/scalable/apps/unity-mail.svg'))
        oAboutDialog.get_content_area().reparent(oBox)
        oAboutDialog.get_content_area().set_hexpand(True)

        lstChildren = oAboutDialog.action_area.get_children()

        for oWidget in lstChildren:

            if (isinstance(oWidget, Gtk.Button) and not isinstance(oWidget, Gtk.ToggleButton)) or (isinstance(oWidget, Gtk.ToggleButton) and len(lstChildren) == 3 and oWidget == lstChildren[1]):

                oWidget.set_property('no-show-all', True)
                oWidget.set_property('visible', False)

        return oBox

    def onFolderActivated(self, oView, nRow, oColumn):

        self.oListStore[nRow][2] = not self.oListStore[nRow][2]
        self.updateUI()

    def updateUI(self):

        bHasFolderSelection = [oRow for oRow in self.oListStore if oRow[2]]
        bHasAccountData = all([oWidget.get_text() for oWidget in [self.EntryHost, self.EntryPort, self.EntryLogin, self.EntryPassword]])
        bHasMultipleAccounts = len(self.lstDicts) > 1

        self.oButtonConnect.set_sensitive(bHasAccountData)
        self.oButtonApply.set_sensitive(bHasFolderSelection and bHasAccountData)
        self.rmbtn.set_sensitive(bHasMultipleAccounts)
        self.addbtn.set_sensitive(bHasFolderSelection and bHasAccountData)
        self.sb.set_sensitive(bHasFolderSelection and bHasAccountData and bHasMultipleAccounts)

    def run(self):

        if not self.lstDicts[0]['Passwd']:
            self.updateEntries()

        self.updateUI()
        Gtk.main()

    def saveAllSettings(self):

        g_oSettings.set_int('max-item-count', self.nMaxCount)
        g_oSettings.set_boolean('enable-notifications', self.notifyswitch.get_active())
        g_oSettings.set_boolean('enable-sound', self.sndswitch.get_active())
        g_oSettings.set_boolean('hide-messages-count', self.hcswitch.get_active())
        g_oSettings.set_string('exec-on-receive', self.commandchooser.getFilename())
        g_oSettings.set_string('custom-sound', self.sndchooser.getFilename())
        g_oSettings.set_boolean('merge-messages', self.oSwitchMerge.get_active())
        g_oSettings.set_enum('message-action', int(self.oComboMessageAction.get_active_id()))

        for strURL, oEntry in (('home', self.oEntryHome), ('compose', self.oEntryCompose), ('inbox', self.oEntryInbox), ('sent', self.oEntrySent)):
            g_oSettings.set_string(strURL, oEntry.getText())

    def onAddAccount(self, btn):

        self.updateAccounts()
        self.lstDicts.append({'Host': 'imap.gmail.com', 'Port': '993', 'Login': '', 'Passwd': '', 'Folders': 'INBOX'})
        self.sb.set_range(1, len(self.lstDicts))
        self.sb.set_value(len(self.lstDicts))
        self.updateUI()

    def onRemoveAccount(self, btn):

        self.updateAccounts()
        index = self.sb.get_value_as_int()-1

        del self.lstDicts[index]

        self.bInit = False
        self.sb.set_range(1, len(self.lstDicts))
        self.bInit = True
        self.updateEntries()
        self.updateUI()

        if index+1 > len(self.lstDicts):
            self.sb.set_value(index)

        self.updateUI()

    def setAccounts(self, lstDicts):

        self.bInit = False
        self.lstDicts = [dct for dct in lstDicts]
        self.nIndex = 0
        self.sb.set_range(1, len(lstDicts))
        self.updateEntries()
        self.updateUI()
        self.bInit = True

    def onAccountChanged(self, sb):

        self.updateAccounts()
        self.nIndex = self.sb.get_value_as_int() - 1
        self.updateEntries()
        self.updateUI()

    def onSoundSwitchActivate(self, sndswitch, param):

        self.sndbox.set_sensitive(sndswitch.get_active())

    def updateEntries(self):

        if self.lstDicts:

            nIndex = self.sb.get_value_as_int() - 1
            self.EntryHost.set_text(self.lstDicts[nIndex]['Host'])
            self.EntryPort.set_text(str(self.lstDicts[nIndex]['Port']))
            self.EntryLogin.set_text(self.lstDicts[nIndex]['Login'])
            self.EntryPassword.set_text(self.lstDicts[nIndex]['Passwd'])

            self.oListStore.clear()

            if self.lstDicts[nIndex]['Passwd']:

                self.getFolderList(False)

                lstFolders = self.lstDicts[nIndex]['Folders'].split('\t')

                for oRow in self.oListStore:
                    oRow[2] = oRow[0] in lstFolders

    def updateAccounts(self):

        if self.bInit:

            self.lstDicts[self.nIndex]['Host'] = self.EntryHost.get_text()

            if self.EntryPort.get_text():
                self.lstDicts[self.nIndex]['Port'] = self.EntryPort.get_text()
            else:
                self.lstDicts[self.nIndex]['Port'] = '993'

            self.lstDicts[self.nIndex]['Login'] = self.EntryLogin.get_text()
            self.lstDicts[self.nIndex]['Passwd'] = self.EntryPassword.get_text()
            self.lstDicts[self.nIndex]['Folders'] = '\t'.join([oRow[0] for oRow in self.oListStore if oRow[2]])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Unity Mail, the main application class
# Authors: Dmitry Shachnev <mitya57@gmail.com>
#          Robert Tari <robert.tari@gmail.com>
# License: GNU GPL 3 or higher; http://www.gnu.org/licenses/gpl.html

APPVERSION = '1.7.5'

import gi

gi.require_version('Gtk', '3.0')
gi.require_version('Notify', '0.7')

import email
import email.errors
import email.header
import email.utils
import dbus
import dbus.service
import logging
import secretstorage
import subprocess
import sys
import time
import re
import UnityMail.imaplib2 as imaplib
import os.path
import os
from gi.repository import GLib, Gtk, Notify
from socket import error as socketerror
from dbus.mainloop.glib import DBusGMainLoop
from UnityMail import g_oTranslation, MessagingMenu, g_oSettings, openURLOrCommand, g_dctDefaultURLs
from UnityMail.idler import Idler
from UnityMail.config import ConfigFile, config_file_path
from UnityMail.dialog import PreferencesDialog, MESSAGEACTION
from UnityMail.actions import DialogActions

imaplib._MAXLINE = 500000#160000 # See discussion in LP: #1309566
logger = logging.getLogger('Unity Mail')
logger.setLevel(logging.DEBUG)
handler = logging.StreamHandler()
handler.setFormatter(logging.Formatter('%(name)s: %(levelname)s: %(message)s'))
logger.addHandler(handler)
logger.propagate = False
m_reThrid = re.compile(b'THRID (\\d+)')
fixFormat = lambda string: string.replace('%(t0)s', '{t0}').replace('%(t1)s', '{t1}')

def decodeWrapper(header):

    # Decodes an Email header, returns a string
    # A hack for headers without a space between decoded name and email
    try:

        dec = email.header.decode_header(header.replace('=?=<', '=?= <'))

    except email.errors.HeaderParseError:

        logger.warning('Exception in decode, skipping')
        return header

    parts = []

    for dec_part in dec:

        if dec_part[1]:

            try:
                parts.append(dec_part[0].decode(dec_part[1]))
            except (AttributeError, LookupError, UnicodeDecodeError):
                logger.warning('Exception in decode, skipping')

        elif isinstance(dec_part[0], bytes):

            parts.append(dec_part[0].decode())

        else:
            parts.append(dec_part[0])

    return str.join(' ', parts)

def getHeaderWrapper(message, header_name, decode):

    header = message[header_name]

    if isinstance(header, str):

        header = header.replace(' \r\n', '').replace('\r\n', '')
        return (decodeWrapper(header) if decode else header)

    return ''

def getSenderName(sender):

    # Strips address, and returns only name
    sname = email.utils.parseaddr(sender)[0]

    return sname if sname else sender

class SessionBus(dbus.service.Object):

    fnSettings = None
    fnClear = None
    fnIsInit = None

    def __init__(self, fnSettings, fnClear, fnIsInit):

        oBusName = dbus.service.BusName('in.tari.unitymail', bus=dbus.SessionBus())
        dbus.service.Object.__init__(self, oBusName, '/in/tari/unitymail')

        self.fnSettings = fnSettings
        self.fnClear = fnClear
        self.fnIsInit = fnIsInit

    @dbus.service.method('in.tari.unitymail')
    def settings(self):
        self.fnSettings()

    @dbus.service.method('in.tari.unitymail')
    def clear(self):
        self.fnClear()

    @dbus.service.method('in.tari.unitymail')
    def isinit(self):
        return self.fnIsInit()

class Connection(object):

    def __init__(self, host, port, login, passwd, folder):

        self.strHost = host
        self.nPort = port
        self.strLogin = login
        self.strPasswd = passwd
        self.oImap = None
        self.lstNotificationQueue = []
        self.oIdler = None
        self.strFolder = folder

class Message(object):

    def __init__(self, oConnection, strUId, title, message_id, timestamp, strSender, thread_id=None):

        self.oConnection = oConnection
        self.strUId = strUId
        self.title = title
        self.message_id = message_id
        self.timestamp = timestamp
        self.thread_id = thread_id
        self.strSender = strSender

class UnityMail(object):

    def __init__(self):

        self.dlgSettings = None
        self.nLastMailTimestamp = 0
        self.first_run = True
        self.lstAccounts = []
        self.lstConnections = []
        self.lstUnreadMessages = []
        self.bDrawAttention = False
        self.bIdlerRunning = False
        self.bReconnecting = False
        self.oMessagingMenu = MessagingMenu(self.onMenuItemClicked, self.openDialog, self.updateMessageAges)

        self.initKeyring()
        self.initConfig()
        DBusGMainLoop(set_as_default=True)
        SessionBus(self.openDialog, self.clear, lambda: bool(self.lstAccounts))
        Notify.init('Unity Mail')
        GLib.set_application_name('Unity Mail')

        if not self.bIdlerRunning:

            GLib.timeout_add_seconds(5, self.check, self.lstConnections)
            self.bIdlerRunning = True

        try:
            GLib.MainLoop().run()
        except KeyboardInterrupt:
            self.close(0)

    def clear(self):

        # WARNING: loadDataFromDicts also calls this!
        if self.lstAccounts:

            for oMessage in self.lstUnreadMessages:

                #self.markMessageAsRead(message)
                self.oMessagingMenu.remove(oMessage.message_id)

            self.setLauncherCount(0)

    def closeConnections(self):

        for oConnection in self.lstConnections:

            if oConnection.oIdler:

                oConnection.oIdler.stop()
                oConnection.oIdler.join()
                oConnection.oIdler = None

            if oConnection.oImap:

                try:
                    oConnection.oImap.close()
                except imaplib.IMAP4.error as oError:
                    pass

                oConnection.oImap.logout()
                oConnection.oImap = None

        self.lstConnections = []
        self.bIdlerRunning = False

    def close(self, nCode):

        self.oMessagingMenu.close()
        self.closeConnections()
        print()
        sys.exit(nCode)

    def onMenuItemClicked(self, strId):

        for message in self.lstUnreadMessages:

            if message.message_id == strId:

                if self.nMessageAction == MESSAGEACTION['MARK']:

                    self.markMessageAsRead(message)

                elif self.nMessageAction == MESSAGEACTION['ASK']:

                    dlg = DialogActions(message.strSender, message.title)
                    nResponse = dlg.run()
                    dlg.destroy()

                    if nResponse == 100:

                        try:

                            if any(s in message.oConnection.strHost for s in ['gmail', 'google']):

                                message.oConnection.oImap.uid('STORE', message.strUId, '+X-GM-LABELS', '\\Trash')

                            else:

                                message.oConnection.oImap.uid('STORE', message.strUId, '+FLAGS', '\\Deleted')
                                message.oConnection.oImap.expunge()

                        except (imaplib.IMAP4.error, socketerror) as oError:

                            logger.error(str(oError))

                    elif nResponse == 200:

                        self.markMessageAsRead(message)

                    elif nResponse == 300:

                        openURLOrCommand('Inbox', message.thread_id)

                    else:

                        self.bDrawAttention = not message.timestamp < self.nLastMailTimestamp
                        self.appendToIndicator(message)
                        self.bDrawAttention = False
                        return False

                else:

                    openURLOrCommand('Inbox', message.thread_id)

        # True removes the message from Appindicator3
        return True

    def markMessageAsRead(self, message):

        # Mark entire conversation
        lstIndexes = [message.strUId]

        if message.thread_id and self.bMergeConversation:

            lstSearch = []

            try:

                lstSearch = message.oConnection.oImap.uid('SEARCH', None, '(X-GM-THRID ' + str(int(message.thread_id, 16)) + ')')

            except imaplib.IMAP4.error as oError:

                logger.error(str(oError))
                return

            if lstSearch[1][0] is not None:
                lstIndexes = [m for m in lstSearch[1][0].split()]

        for strIndex in lstIndexes:

            try:
                message.oConnection.oImap.uid('STORE', strIndex, '+FLAGS', '\\Seen')
            except (imaplib.IMAP4.error, socketerror) as e:
                logger.error(str(e))

    def initConfig(self):

        # REMOVE THIS BLOCK IN 1.8
        if os.path.isfile(config_file_path):

            config = ConfigFile(config_file_path)
            self.nMaxCount = config.get_int_with_default('MaxItemCount', 20)
            self.bEnableNotifications = config.get_bool_with_default('EnableNotifications', True)
            self.bPlaySound = config.get_bool_with_default('EnableSound', True)
            self.bHideCount = config.get_bool_with_default('HideMessagesCount', True)
            self.strCommand = config.get_str_with_default('ExecOnReceive')
            self.custom_sound = config.get_str_with_default('CustomSound')
            self.bMergeConversation = config.get_bool_with_default('MergeMessages', False)
            self.nMessageAction = config.get_int_with_default('MessageAction', MESSAGEACTION['OPEN'])
            self.on_click_urls = {}

            for urlname in 'Home', 'Compose', 'Inbox', 'Sent':
                self.on_click_urls[urlname] = config.get_str_with_default(urlname, g_dctDefaultURLs[urlname], section = 'URLs')

            g_oSettings.set_int('max-item-count', self.nMaxCount)
            g_oSettings.set_boolean('enable-notifications', self.bEnableNotifications)
            g_oSettings.set_boolean('enable-sound', self.bPlaySound)
            g_oSettings.set_boolean('hide-messages-count', self.bHideCount)
            g_oSettings.set_string('exec-on-receive', self.strCommand)
            g_oSettings.set_string('custom-sound', self.custom_sound)
            g_oSettings.set_boolean('merge-messages', self.bMergeConversation)
            g_oSettings.set_enum('message-action', self.nMessageAction)
            g_oSettings.set_string('home', self.on_click_urls['Home'])
            g_oSettings.set_string('compose', self.on_click_urls['Compose'])
            g_oSettings.set_string('inbox', self.on_click_urls['Inbox'])
            g_oSettings.set_string('sent', self.on_click_urls['Sent'])

            os.remove(config_file_path)

        else:

            self.nMaxCount = g_oSettings.get_int('max-item-count')
            self.bEnableNotifications = g_oSettings.get_boolean('enable-notifications')
            self.bPlaySound = g_oSettings.get_boolean('enable-sound')
            self.bHideCount = g_oSettings.get_boolean('hide-messages-count')
            self.strCommand = g_oSettings.get_string('exec-on-receive')
            self.custom_sound = g_oSettings.get_string('custom-sound')
            self.bMergeConversation = g_oSettings.get_boolean('merge-messages')
            self.nMessageAction = g_oSettings.get_enum('message-action')
            self.on_click_urls = {}
            self.on_click_urls['Home'] = g_oSettings.get_string('home')
            self.on_click_urls['Compose'] = g_oSettings.get_string('compose')
            self.on_click_urls['Inbox'] = g_oSettings.get_string('inbox')
            self.on_click_urls['Sent'] = g_oSettings.get_string('sent')

    def initKeyring(self):

        bus = secretstorage.dbus_init()

        try:

            self.collection = secretstorage.get_default_collection(bus)
            self.collection.is_locked()

        except secretstorage.SecretStorageException as e:

            logger.critical(str(e))
            self.close(1)

        if self.collection.is_locked():
            self.collection.unlock()

        if self.collection.is_locked():

            logger.critical('Failed to unlock the collection, exiting')
            self.close(1)

        self.mail_keys = list(self.collection.search_items({'application': 'unity-mail'}))

        if not self.mail_keys:
            self.openDialog()

        if not self.lstAccounts:

            for key in sorted(self.mail_keys, key=lambda item: item.item_path):

                dctAttributes = key.get_attributes()
                strHost = dctAttributes['server']
                nPort = int(dctAttributes['port'])
                strLogin = dctAttributes['username']
                strPasswd = key.get_secret().decode('utf-8')
                strFolders = 'INBOX'

                try:
                    strFolders = dctAttributes['folders']
                except KeyError:
                    pass

                self.lstAccounts.append({'Host': strHost, 'Port': nPort, 'Login': strLogin, 'Passwd': strPasswd, 'Folders': strFolders})

                for strFolder in strFolders.split('\t'):
                    self.lstConnections.append(Connection(strHost, nPort, strLogin, strPasswd, strFolder))

    def createKeyringItem(self, ind, update=False):

        strFolders = 'INBOX'

        try:
            strFolders = self.lstAccounts[ind]['Folders']
        except KeyError:
            pass

        attrs = {'application': 'unity-mail', 'service': 'imap', 'server': self.lstAccounts[ind]['Host'], 'port': str(self.lstAccounts[ind]['Port']), 'username': self.lstAccounts[ind]['Login'], 'folders': strFolders}
        label = 'unity-mail: ' + self.lstAccounts[ind]['Login'] + ' at ' + self.lstAccounts[ind]['Host']

        if update:

            self.mail_keys[ind].set_attributes(attrs)
            self.mail_keys[ind].set_secret(self.lstAccounts[ind]['Passwd'])
            self.mail_keys[ind].set_label(label)

        else:

            self.collection.unlock()
            self.collection.create_item(label, attrs, self.lstAccounts[ind]['Passwd'], True)

    def openDialog(self):

        if not self.dlgSettings:

            self.dlgSettings = PreferencesDialog()
            self.dlgSettings.connect('response', self.onDialogResponse)

            if self.mail_keys:
                self.dlgSettings.setAccounts(self.lstAccounts)

            self.dlgSettings.run()
            self.dlgSettings.destroy()
            self.dlgSettings = None

    def onDialogResponse(self, dlg, response):

        if response == Gtk.ResponseType.APPLY:

            dlg.updateAccounts()
            dlg.saveAllSettings()

            self.lstAccounts = dlg.lstDicts

            self.loadDataFromDicts()

            for index in range(len(self.lstAccounts), len(self.mail_keys)):

                # Remove old keys
                self.mail_keys[index].delete()

            for index in range(len(self.lstAccounts)):

                # Create new keys or update existing
                self.createKeyringItem(index, update=(index < len(self.mail_keys)))

            self.mail_keys = list(self.collection.search_items({'application': 'unity-mail'}))

            if self.mail_keys and self.lstAccounts and self.lstConnections and not self.bIdlerRunning:

                self.bIdlerRunning = True
                GLib.timeout_add_seconds(5, self.check, self.lstConnections)

        if response in [Gtk.ResponseType.APPLY, Gtk.ResponseType.CANCEL]:
            dlg.destroy()

    def loadDataFromDicts(self):

        self.closeConnections()
        self.initConfig()
        self.clear()
        self.lstUnreadMessages = []
        self.nLastMailTimestamp = 0
        self.first_run = True

        for dct in self.lstAccounts:

            strFolders = 'INBOX'

            try:
                strFolders = dct['Folders']
            except KeyError:
                pass

            nPort = 993

            try:
                nPort = int(dct['Port'])
            except ValueError:
                pass

            for strFolder in strFolders.split('\t'):
                self.lstConnections.append(Connection(dct['Host'], nPort, dct['Login'], dct['Passwd'], strFolder))

    def establishConnection(self, oConnection):

        try:

            oConnection.oImap = imaplib.IMAP4_SSL(oConnection.strHost, oConnection.nPort)

        except Exception:

            oConnection.oImap = imaplib.IMAP4(oConnection.strHost, oConnection.nPort)
            oConnection.oImap.starttls()

    def tryEstablishConnection(self, oConnection):

        try:
            self.establishConnection(oConnection)
        except Exception:
            return False

        try:

            oConnection.oImap.login(oConnection.strLogin, oConnection.strPasswd)

        except (imaplib.IMAP4.error, UnicodeEncodeError) as e:

            oNotification = Notify.Notification.new(_('Invalid account data'), '', 'unity-mail')
            oNotification.set_property('body', _('Unable to connect to account "{accountName}", the application will now exit.').format(accountName=oConnection.strLogin) + '\n\n' + _('You can run "{command}" to delete all your login settings.').format(command='unity-mail-reset'))
            oNotification.set_hint('desktop-entry', GLib.Variant.new_string('unity-mail'))
            oNotification.set_timeout(Notify.EXPIRES_NEVER)
            oNotification.show()
            self.close(1)

        return True

    def appendToIndicator(self, message):

        if self.oMessagingMenu.hasSource(message.message_id):
            return

        title = message.title

        if len(title) > 50:
            title = title[:50] + '...'

        if len(self.lstAccounts) > 1:
            title = '↪ ' + message.oConnection.strLogin + '\n' + title

        self.oMessagingMenu.append(message.message_id, title, message.timestamp, self.bDrawAttention)

    def encodeFolderName(self, folder):

        folder_name_utf7 = bytes(folder, 'utf-7').replace(b'+', b'&').replace(b' &', b'- &')

        if b' ' in folder_name_utf7:
            folder_name_utf7 = b'"' + folder_name_utf7 + b'"'

        return folder_name_utf7

    def onIdle(self, oConnection):

        lstMessages = []
        lstNewMessages = []
        lstUnread = []

        try:

            search = oConnection.oImap.uid('SEARCH', '(UNSEEN)')

        except imaplib.IMAP4.abort as oError:

            if not self.bReconnecting:

                logger.info('Connection to "{0}:{1}" closed. Trying to reconnect.'.format(oConnection.strLogin, oConnection.strFolder))
                GLib.timeout_add_seconds(5, self.check, [oConnection])
                self.bReconnecting = True

            return

        if search[1][0] is not None:
            lstMessages = search[1][0].split()

        for m in lstMessages[-self.nMaxCount:]:

            typ = None
            msg_data = None
            thread_id = None
            msg = None

            try:

                typ, msg_data = oConnection.oImap.uid('FETCH', m, '(X-GM-THRID BODY.PEEK[HEADER.FIELDS (DATE SUBJECT FROM MESSAGE-ID)])')

                for lstField in msg_data:

                    if 'THRID' in str(lstField):

                        thread_id = '%x' % int(m_reThrid.search(lstField[0]).group(1))
                        break

            except imaplib.IMAP4.error:

                typ, msg_data = oConnection.oImap.uid('FETCH', m, '(BODY.PEEK[HEADER.FIELDS (DATE SUBJECT FROM MESSAGE-ID)])')
                thread_id = None

            for response_part in msg_data:

                if isinstance(response_part, tuple):
                    msg = email.message_from_bytes(response_part[1])

            if msg is None:
                continue

            message_id = msg['Message-Id']

            if not isinstance(message_id, str):
                message_id = oConnection.strHost + ':' + oConnection.strLogin + ':' + oConnection.strFolder + ':' + str(m.decode())

            bMessageExists = False

            for cMessage in self.lstUnreadMessages:

                if cMessage.message_id == message_id:

                    bMessageExists = True
                    break

            if not bMessageExists:

                sender = getHeaderWrapper(msg, 'From', True)
                subj = getHeaderWrapper(msg, 'Subject', True)
                date = getHeaderWrapper(msg, 'Date', False)

                try:

                    tuple_time = email.utils.parsedate_tz(date)
                    timestamp = email.utils.mktime_tz(tuple_time)

                    if timestamp > time.time():

                        # Message time is larger than the current one
                        timestamp = time.time()

                except TypeError:

                    # Failed to get time from message
                    timestamp = time.time()

                # Number of seconds to number of microseconds
                timestamp *= (10**6)

                while subj.lower().startswith('re:'):
                    subj = subj[3:]

                while subj.lower().startswith('fwd:'):
                    subj = subj[4:]

                subj = subj.strip()

                if sender.startswith('"'):

                    pos = sender[1:].find('"')

                    if pos >= 0:
                        sender = sender[1:pos+1]+sender[pos+2:]

                ilabel = subj if subj else _('No subject')

                # Display only last message in thread
                bConversationInUnread = False
                bConversationInNew = False

                if thread_id and self.bMergeConversation:

                    for oMessage in self.lstUnreadMessages:

                        if oMessage.thread_id == thread_id:

                            oMessage.timestamp = max(timestamp, oMessage.timestamp)
                            bConversationInUnread = True
                            break

                    if not bConversationInUnread:

                        for oMessage in lstNewMessages:

                            if oMessage.thread_id == thread_id:

                                oMessage.timestamp = max(timestamp, oMessage.timestamp)
                                bConversationInNew = True
                                break

                if not bConversationInUnread and not bConversationInNew:

                    message = Message(oConnection, m, ilabel, message_id, timestamp, sender, thread_id)
                    lstNewMessages.append(message)

                if timestamp > self.nLastMailTimestamp:

                    if self.bEnableNotifications:

                        if not bConversationInNew:

                            oConnection.lstNotificationQueue.append([sender, subj, oConnection, thread_id])

                        else:

                            for lstNotification in oConnection.lstNotificationQueue:

                                if lstNotification[3] == thread_id:

                                    lstNotification[0] = sender
                                    break

                    self.nLastMailTimestamp = timestamp
                    self.bDrawAttention = True

                if self.strCommand and not self.first_run:

                    try:

                        subprocess.call((self.strCommand, sender, ilabel))

                    except OSError as e:

                        # File doesn't exist or is not executable
                        logger.warning('Cannot execute command: {0}'.format(str(e)))

            else:

                lstUnread.append(message_id)

        self.updateIndicator(lstNewMessages, [oMessage for oMessage in self.lstUnreadMessages if oMessage.oConnection == oConnection and oMessage.message_id not in lstUnread], oConnection)

    def updateIndicator(self, lstNewMessages, lstRemovedMessages, oConnection):

        for oMessage in [oMessage for oMessage in self.lstUnreadMessages if oMessage.oConnection == oConnection]:

            # Removed outside the app
            if oMessage in lstRemovedMessages:
                self.oMessagingMenu.remove(oMessage.message_id)

            # Cleared
            if not self.oMessagingMenu.hasSource(oMessage.message_id) and oMessage not in lstNewMessages:

                self.markMessageAsRead(oMessage)
                self.lstUnreadMessages.remove(oMessage)

        self.lstUnreadMessages = sorted(self.lstUnreadMessages + lstNewMessages, key=lambda m: m.timestamp)[-self.nMaxCount:]

        #logger.debug('Unread: {0}, New: {1}, Removed: {2}'.format(len(self.lstUnreadMessages), len(lstNewMessages), len(lstRemovedMessages)))

        if lstNewMessages:

            for cMessage in lstNewMessages:
                self.appendToIndicator(cMessage)

        try:

            if self.first_run and oConnection != self.lstConnections[-1]:

                pass

            elif self.first_run and oConnection == self.lstConnections[-1]:

                self.showNotifications()
                self.first_run = False

            elif lstNewMessages:

                self.showNotifications()

        except GLib.GError as e:

            logger.warning(str(e))

        self.setLauncherCount(len(self.lstUnreadMessages))

        if not self.first_run:
            self.bDrawAttention = False

    def updateMessageAges(self):

        for oMessage in self.lstUnreadMessages:
            self.oMessagingMenu.update(oMessage.message_id, oMessage.timestamp)

        return True

    def check(self, lstConnections):

        for oConnection in lstConnections:

            if oConnection.oIdler:

                oConnection.oIdler.stop()
                oConnection.oIdler.join()
                oConnection.oIdler = None

            if oConnection.oImap:

                try:
                    oConnection.oImap.close()
                except imaplib.IMAP4.error as oError:
                    pass

                oConnection.oImap.logout()
                oConnection.oImap = None

            try:

                if not self.tryEstablishConnection(oConnection):
                    continue

                if oConnection.oImap.select(mailbox=self.encodeFolderName(oConnection.strFolder))[0] != 'OK':

                    logger.error('Mailbox "{0}:{1}" does not exist'.format(oConnection.strLogin, oConnection.strFolder))

                    if oConnection.oImap:

                        try:
                            oConnection.oImap.close()
                        except imaplib.IMAP4.error:
                            pass

                        oConnection.oImap.logout()
                        oConnection.oImap = None

                    continue

                logger.info('Connection to "{0}:{1}" established'.format(oConnection.strLogin, oConnection.strFolder))
                self.onIdle(oConnection)
                oConnection.oIdler = Idler(oConnection, self.onIdle)
                oConnection.oIdler.start()

            except (imaplib.IMAP4.error, socketerror) as e:

                logger.error('Error in connection "{0}:{1}": {2}'.format(oConnection.strLogin, oConnection.strFolder, str(e)))
                oConnection.oImap = None

            except KeyboardInterrupt:

                oConnection.oImap = None
                self.close(0)

        self.bReconnecting = False

        return False

    def setLauncherCount(self, nCount):

        self.oMessagingMenu.setCount(nCount, any([oImap for oImap in self.lstConnections]))

    def showNotifications(self):

        lstNotificationsQueue = []

        for oConnection in self.lstConnections:

            lstNotificationsQueue += oConnection.lstNotificationQueue
            oConnection.lstNotificationQueue = []

        number_of_mails = len(lstNotificationsQueue)
        basemessage = g_oTranslation.ngettext('You have %d unread mail', 'You have %d unread mails', number_of_mails)
        basemessage = basemessage.replace('%d', '{0}')

        if number_of_mails and self.bPlaySound:

            try:

                if self.custom_sound:
                    subprocess.call(('canberra-gtk-play', '-f', self.custom_sound))
                else:
                    subprocess.call(('canberra-gtk-play', '-i', 'message-new-email'))

            except OSError as e:

                logger.warning(str(e))

        if number_of_mails > 1:

            senders = set(getSenderName(lstNotification[0]) for lstNotification in lstNotificationsQueue)
            unknown_sender = ('' in senders)

            if unknown_sender:
                senders.remove('')

            ts = tuple(senders)

            if len(ts) > 2 or (len(ts) == 2 and unknown_sender):
                message = fixFormat(_('from %(t0)s, %(t1)s and others')).format(t0=ts[0], t1=ts[1])
            elif len(ts) == 2 and not unknown_sender:
                message = fixFormat(_('from %(t0)s and %(t1)s')).format(t0=ts[0], t1=ts[1])
            elif len(ts) == 1 and not unknown_sender:
                message = _('from %s').replace('%s', '{0}').format(getSenderName(ts[0]))
            else:
                message = None

            oNotification = Notify.Notification.new(basemessage.format(number_of_mails), message, 'unity-mail')
            oNotification.set_hint('desktop-entry', GLib.Variant.new_string('unity-mail'))
            oNotification.show()

        elif number_of_mails:

            lstNotification = lstNotificationsQueue[0]

            if lstNotification[0]:
                message = _('New mail from %s').replace('%s', '{0}').format(getSenderName(lstNotification[0]))
            else:
                message = basemessage.format(1)

            oNotification = Notify.Notification.new(message, lstNotification[1], 'unity-mail')
            oNotification.set_hint('desktop-entry', GLib.Variant.new_string('unity-mail'))
            oNotification.show()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import UnityMail.imaplib2 as imaplib
from threading import *

class Idler(object):

    def __init__(self, oConnection, fnCallback):

        self.oThread = Thread(target=self.idle)
        self.oConnection = oConnection
        self.oEvent = Event()
        self.fnCallback = fnCallback
        self.bNeedSync = False

    def start(self):

        self.oThread.start()

    def stop(self):

        self.oEvent.set()

    def join(self):

        self.oThread.join()

    def idle(self):

        while True:

            if self.oEvent.isSet():
                return

            self.bNeedSync = False

            def callback(lstArgs):

                if not self.oEvent.isSet():

                    self.bNeedSync = True
                    self.oEvent.set()

            try:

                self.oConnection.oImap.idle(callback=callback)

            except imaplib.IMAP4.abort as oError:

                self.fnCallback(self.oConnection)
                return

            self.oEvent.wait()

            if self.bNeedSync:

                self.oEvent.clear()
                self.fnCallback(self.oConnection)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Unity Mail, message actions dialog
# Authors: Robert Tari <robert.tari@gmail.com>
# License: GNU GPL 3 or higher; http://www.gnu.org/licenses/gpl.html

from gi.repository import Gtk

class DialogActions(Gtk.Dialog):

    def __init__(self, strSender, strSubject):

        Gtk.Dialog.__init__(self, _('Message actions'), None, 0, (_('Delete'), 100, _('Mark as read'), 200, _('Open in browser'), 300))
        self.set_icon_name('unity-mail')
        self.set_position(Gtk.WindowPosition.CENTER)
        oImage = Gtk.Image.new_from_icon_name('unity-mail', Gtk.IconSize.DIALOG)
        oImage.props.valign = Gtk.Align.START
        oGrid = Gtk.Grid(border_width=10, row_spacing=2, column_spacing=10)
        oGrid.attach(oImage, 0, 0, 1, 4)
        oGrid.attach(Gtk.Label('<b>' + _('Sender') + '</b>', xalign=0, use_markup=True), 1, 1, 1, 1)
        oGrid.attach(Gtk.Label(strSender, xalign=0, margin_bottom=5), 1, 2, 1, 1)
        oGrid.attach(Gtk.Label('<b>' + _('Subject') + '</b>', xalign=0, use_markup=True, margin_bottom=5), 1, 3, 1, 1)
        oGrid.attach(Gtk.Label(strSubject, xalign=0, margin_bottom=5), 1, 4, 1, 1)
        self.get_content_area().add(oGrid)
        oButtonDelete = self.get_widget_for_response(100)
        oButtonDelete.set_image(Gtk.Image.new_from_icon_name('gtk-delete', Gtk.IconSize.BUTTON))
        oButtonMark = self.get_widget_for_response(200)
        oButtonMark.set_image(Gtk.Image.new_from_icon_name('gtk-ok', Gtk.IconSize.BUTTON))
        oButtonOpen = self.get_widget_for_response(300)
        oButtonOpen.set_image(Gtk.Image.new_from_icon_name('web-browser', Gtk.IconSize.BUTTON))
        self.set_keep_above(True)
        self.show_all()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Unity Mail, common initialization code
# Authors: Dmitry Shachnev <mitya57@gmail.com>
#          Robert Tari <robert.tari@gmail.com>
# License: GNU GPL 3 or higher; http://www.gnu.org/licenses/gpl.html

import gi

gi.require_version('Gtk', '3.0')

import gettext
import os
import gi
import platform
import psutil
import subprocess
import importlib
import locale
import time
import webbrowser
from babel.dates import format_timedelta
from gi.repository import Gio, Gtk, GLib

try:
    g_oTranslation = gettext.translation('unity-mail')
except IOError:
    g_oTranslation = gettext.NullTranslations()

g_oTranslation.install()

g_dctDefaultURLs = {'Home': 'https://mail.google.com/mail/', 'Compose': 'https://mail.google.com/mail/#compose', 'Inbox': 'https://mail.google.com/mail/#inbox', 'Sent': 'https://mail.google.com/mail/#sent'}
g_oSettings = Gio.Settings.new('in.tari.unity-mail')

def isRunning():

    nCount = 0

    for oProc in psutil.process_iter():

        strName = oProc.name

        if not isinstance(strName, str):
           strName = oProc.name()

        if strName == 'python3':

            lstCmdline = oProc.cmdline

            if not isinstance(lstCmdline, list):
               lstCmdline = oProc.cmdline()

            for strCmd in lstCmdline:

                if strCmd.endswith('unity-mail'):
                    nCount += 1

        elif strName.endswith('unity-mail'):

            nCount += 1

    return nCount

def printNames():

    print('Possible URL names: "Home", "Compose", "Inbox", "Sent"')

def openURLOrCommand(strName, strThread=None):

    if strName in g_dctDefaultURLs:

        strURL = g_oSettings.get_string(strName.lower())

        if strURL.startswith('Exec:'):

            subprocess.Popen(strURL[5:], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True)

        else:

            if strThread:

                if not strURL.endswith('/'):
                    strURL += '/'

                strURL += strThread

            #subprocess.Popen(['xdg-open', strURL], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
            webbrowser.open_new_tab(strURL)

    else:

        print('Unknown URL name!')
        printNames()

class MenuItemMail(Gtk.MenuItem):

    strId = None

    def __init__(self, strId):

        self.strId = strId

        Gtk.MenuItem.__init__(self)

class MessagingMenu(object):

    oUnity = None
    oMessagingMenu = None
    oAppIndicator = None

    def __init__(self, fnActivate, fnSettings, fnUpdateMessageAges):

        self.fnActivate = fnActivate
        self.launcher = None
        self.nMenuItems = 0
        self.nMessageAgeTimer = None
        self.oMenuItemClear = None
        self.oMailIcon = Gio.Icon.new_for_string('mail-unread')

        strDesktop = os.environ.get('XDG_CURRENT_DESKTOP').lower()
        nVersion = int(platform.linux_distribution()[1].replace('.', ''))

        if 'unity' in strDesktop or ('mate' in strDesktop and nVersion >= 1704):

            gi.require_version('Unity', '7.0')
            self.oUnity = importlib.import_module('gi.repository.Unity')

            strDesktopId = 'unity-mail.desktop'

            for strId in self.oUnity.LauncherFavorites.get_default().enumerate_ids():

                if 'unity-mail' in strId:

                    strDesktopId = strId
                    break

            self.oLauncher = self.oUnity.LauncherEntry.get_for_desktop_id(strDesktopId)

        if 'unity' in strDesktop or 'xfce' in strDesktop or ('mate' in strDesktop and nVersion >= 1610):

            gi.require_version('MessagingMenu', '1.0')
            self.oMessagingMenu = importlib.import_module('gi.repository.MessagingMenu')

            self.oIndicator = self.oMessagingMenu.App(desktop_id='unity-mail.desktop')
            self.oIndicator.register()
            self.oIndicator.connect('activate-source', lambda a, i: self.onMenuItemClicked(i))

        else:

            gi.require_version('AppIndicator3', '0.1')
            self.oAppIndicator = importlib.import_module('gi.repository.AppIndicator3')

            self.oIndicator = self.oAppIndicator.Indicator.new('unity-mail', 'indicator-messages', self.oAppIndicator.IndicatorCategory.APPLICATION_STATUS)
            self.oIndicator.set_attention_icon('indicator-messages-new')
            self.oIndicator.set_status(self.oAppIndicator.IndicatorStatus.ACTIVE)
            oMenu = Gtk.Menu()
            oMenu.append(Gtk.SeparatorMenuItem())
            oMenuItemInbox = Gtk.MenuItem()
            oMenuItemInbox.connect('activate', lambda w: openURLOrCommand('Home'))
            oBoxInbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 6)
            oBoxInbox.pack_start(Gtk.Image.new_from_stock(Gtk.STOCK_HOME, Gtk.IconSize.MENU), False, False, 0)
            oBoxInbox.pack_start(Gtk.Label(_('Open webmail home page'), xalign=0), True, True, 0)
            oMenuItemInbox.add(oBoxInbox)
            oMenu.append(oMenuItemInbox)
            self.oMenuItemClear = Gtk.MenuItem(sensitive=False)
            self.oMenuItemClear.connect('activate', self.onClear)
            oBoxClear = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 6)
            oBoxClear.pack_start(Gtk.Image.new_from_icon_name('gtk-clear', Gtk.IconSize.MENU), False, False, 0)
            oBoxClear.pack_start(Gtk.Label(_('Clear'), xalign=0), True, True, 0)
            self.oMenuItemClear.add(oBoxClear)
            oMenu.append(self.oMenuItemClear)
            oMenuItemConfig = Gtk.MenuItem()
            oMenuItemConfig.connect('activate', lambda w: fnSettings())
            oBoxConfig = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 6)
            oBoxConfig.pack_start(Gtk.Image.new_from_stock(Gtk.STOCK_PREFERENCES, Gtk.IconSize.MENU), False, False, 0)
            oBoxConfig.pack_start(Gtk.Label(_('Settings'), xalign=0), True, True, 0)
            oMenuItemConfig.add(oBoxConfig)
            oMenu.append(oMenuItemConfig)
            oMenu.show_all()
            self.oIndicator.set_menu(oMenu)
            self.nMenuItems = len(self.oIndicator.get_menu().get_children())
            self.nMessageAgeTimer = GLib.timeout_add_seconds(60, fnUpdateMessageAges)

    def getMessageAge(self, nTimestamp):

        nTimeDelta = int(time.time() - nTimestamp / 1000000)
        strGranularity = 'minute'

        if nTimeDelta > (7 * 24 * 60 * 60):
            strGranularity = 'week'
        elif nTimeDelta > (24 * 60 * 60):
            strGranularity = 'day'
        elif nTimeDelta > (60 * 60):
            strGranularity = 'hour'
        elif nTimeDelta < 60:
            nTimeDelta = 61

        return ' (' + format_timedelta(nTimeDelta, granularity=strGranularity, format='short', locale=locale.getlocale()[0]) + ')'

    def onMenuItemClicked(self, strId):

        if self.fnActivate(strId):

            self.setCount(-1, True)

            if not self.oMessagingMenu:
                self.remove(strId)

    def append(self, strId, strTitle, nTimestamp, bDrawAttention):

        if self.oMessagingMenu:

            self.oIndicator.append_source_with_time(strId, self.oMailIcon, strTitle, nTimestamp)

            if bDrawAttention:
                self.oIndicator.draw_attention(strId)

        else:

            oMenu = self.oIndicator.get_menu()
            oMenuItemMail = MenuItemMail(strId)
            oMenuItemMail.connect('activate', lambda w: self.onMenuItemClicked(w.strId))
            oBox = Gtk.Box(Gtk.Orientation.HORIZONTAL, 6)
            oBox.pack_start(Gtk.Image.new_from_icon_name('mail-unread', Gtk.IconSize.MENU), False, False, 0)
            oBox.pack_start(Gtk.Label(strTitle + self.getMessageAge(nTimestamp), xalign=0), True, True, 0)
            oMenuItemMail.add(oBox)
            oMenu.insert(oMenuItemMail, len(oMenu.get_children()) - self.nMenuItems)
            oMenu.show_all()

            if bDrawAttention:
                self.oIndicator.set_status(self.oAppIndicator.IndicatorStatus.ATTENTION)

            self.oMenuItemClear.set_sensitive(True)

    def remove(self, strId):

        if self.oMessagingMenu:

            if self.oIndicator.has_source(strId):
                self.oIndicator.remove_source(strId)

        else:

            oMenu = self.oIndicator.get_menu()

            for oItem in oMenu.get_children()[0:-self.nMenuItems]:

                if oItem.strId == strId:
                    oMenu.remove(oItem)

            if len(oMenu.get_children()) - self.nMenuItems == 0:

                self.oIndicator.set_status(self.oAppIndicator.IndicatorStatus.ACTIVE)
                self.oMenuItemClear.set_sensitive(False)

    def hasSource(self, strId):

        if self.oMessagingMenu:

            return self.oIndicator.has_source(strId)

        else:

            for oItem in self.oIndicator.get_menu().get_children()[0:-self.nMenuItems]:

                if oItem.strId == strId:
                    return True

        return False

    def close(self):

        if not self.oMessagingMenu:
            GLib.source_remove(self.nMessageAgeTimer)

    def update(self, strId, nTimestamp):

        for oItem in self.oIndicator.get_menu().get_children()[0:-self.nMenuItems]:

            if oItem.strId == strId:

                oLabel = oItem.get_children()[0].get_children()[1]
                GLib.idle_add(oLabel.set_text, oLabel.get_text().rpartition(' (')[0] + self.getMessageAge(nTimestamp))

    def onClear(self, oWidget):

        oMenu = self.oIndicator.get_menu()

        for oItem in oMenu.get_children()[0:-self.nMenuItems]:
            oMenu.remove(oItem)

        if len(oMenu.get_children()) - self.nMenuItems == 0:

            self.oIndicator.set_status(self.oAppIndicator.IndicatorStatus.ACTIVE)
            self.oMenuItemClear.set_sensitive(False)

        self.setCount(0, True)

    def setCount(self, nCount, bVisible):

        if nCount == -1:

            if self.oUnity:
                nCount = self.oLauncher.get_property('count') - 1
            elif not self.oMessagingMenu:
                nCount = int(self.oIndicator.get_label()) - 1

        bVisible = bVisible and ((nCount > 0) or not g_oSettings.get_boolean('hide-messages-count'))

        if self.oUnity:

            GLib.idle_add(self.oLauncher.set_property, 'count', nCount)
            GLib.idle_add(self.oLauncher.set_property, 'count_visible', bVisible)

        elif not self.oMessagingMenu:
            GLib.idle_add(self.oIndicator.set_label, str(nCount) if bVisible else '', '')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from distutils.core import setup
import os

APPVERSION = None

with open(os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__), 'UnityMail/application.py')), 'r') as oFile:

    for strLine in oFile:

        if 'APPVERSION = ' in strLine:

            APPVERSION = strLine.split('=')[1].strip(" '\n")
            break

setup(
    name = 'UnityMail',
    version = APPVERSION,
    description = 'Mail notifications and counter for Unity/MATE/Xfce/LXDE/etc.',
    author = 'Robert Tari',
    author_email = 'robert.tari@gmail.com',
    url = 'https://tari.in/www/software/unitymail/',
    packages = ['UnityMail'],
    scripts = ['unity-mail', 'unity-mail-url', 'unity-mail-clear', 'unity-mail-settings', 'unity-mail-reset'],
    license = 'GPL 3+'
    )

